<?php 
namespace Concrete\Package\CcOptinFree\Controllers;

use Loader;

use Concrete\Core\Http\ResponseAssetGroup;

use Concrete\Package\CcOptinFree\CcOptinDate;
use Concrete\Package\CcOptinFree\Models\CcOptinCampaign;
use Concrete\Package\CcOptinFree\Models\CcOptinType;
use Concrete\Package\CcOptinFree\Models\CcOptinDesign;
use Concrete\Package\CcOptinFree\Models\CcOptinTemplate;

/**
* A somewhat complicated save context because both type and design is saved in the same request.
* The post array and validation needs to be split and merged again afterwards.
*/
class CcOptinSaveContent
{
	private $campaign;
	private $type;
	private $design;
	private $post;
	private $_result;

	public function __construct($campaign, $post) {
		$this->campaign = $campaign;
		$this->type = CcOptinType::create($campaign->typeClass, $campaign->typeID);
		$this->design = CcOptinDesign::create($campaign->designClass, $campaign->designID);
		$this->post = $post;
	}

	public function errors() {
		return $this->_result;
	}

	public function save() {
		$typeResult = array();
		$designResult = array();

		if($this->validate($this->type, 'type', $typeResult) && 
			$this->validate($this->design, 'design', $designResult)) 
		{
			$this->type->save();
			$this->design->save();
			return true;
		}

		// Validation failed, put back the prefix on the results and concatenate them.
		$this->addPrefix($typeResult, 'type');
		$this->addPrefix($designResult, 'design');

		$this->_result = array_merge_recursive($typeResult, $designResult);
		return false;
	}

	private function addPrefix(&$result, $prefix) {
		if(!$result['error']['validation']) return;

		foreach ($result['error']['validation'] as $key => $value) {
			$result['error']['validation'][$prefix . '_' . $key] = $value;
			unset($result['error']['validation'][$key]);
		}
	}

	private function validate($obj, $prefix, &$result) {
		// Save type
		$data = $this->extractData($this->post, $prefix);
		$validator = new CcOptinVal($data, $result);

		$obj->validate($validator);

		return $validator->applyTo($obj);
	}

	private function extractData($post, $prefix) {
		$output = array();
		foreach ($post as $key => $value) {
			$search = $prefix . '_';
			if(strpos($key, $search) === 0)
				$output[substr($key, strlen($search))] = $value;
		}
		return $output;
	}	
}

/**
* Edit the content for a Campaign
*/
class ContentController extends CcOptinController
{
	private $db;

	public function __construct() {
		parent::__construct(__FILE__, CcOptinSecurity::read);
		$this->db = Loader::db();
	}

	protected function post($post) {
		if(!$this->security->canWrite()) {
			return $this->outputJsonError(t("You don't have write access."));
		}

		$campaign = $this->campaign($post);
		$saveContent = new CcOptinSaveContent($campaign, $post);

		$this->outputJson($saveContent->save() ? $campaign : $saveContent->errors());
	}

	protected function get($get) {
		$campaign = $this->campaign($get);
		
		$type = CcOptinType::create($campaign->typeClass, $campaign->typeID);
		$design = CcOptinDesign::create($campaign->designClass, $campaign->designID);
		$template = CcOptinTemplate::get($design->templateHandle);

		$typeRef = new \ReflectionClass($type);
		$designRef = new \ReflectionClass($design);

		// Color picker requires some additions before the view.
		$this->requireAsset('core/colorpicker');
		$this->requireAsset('core/file-manager');
		$this->requireAsset('javascript', 'bootstrap/tooltip');
		$this->requireAsset('redactor');

		$this->view('edit', array(
			'campaign' => $campaign, 'type' => $type, 'design' => $design, 'template' => $template,
			'typeView' => 'types/' . strtolower($typeRef->getShortName()),
			'designView' => 'designs/' . strtolower($designRef->getShortName()),
			'action' => $this->action()
		));
	}

	// ---------------------------------------------

	private function campaign($array) {
		$id = intval($array['id']);

		if(!$id) throw new \Exception("No campaign specified for content controller.");

		$campaign = new CcOptinCampaign();
		$campaign->load('id = ?', $id);

		return $campaign;
	}
}
